@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@ import pytest
 
 import numpy as np
 import piquasso as pq
-import strawberryfields as sf
 
 
 pytestmark = pytest.mark.benchmark(
@@ -36,7 +35,7 @@ def piquasso_benchmark(benchmark, pq_gaussian_simulator, example_pq_gaussian_sta
             )
 
 
-def strawberryfields_benchmark(benchmark, example_sf_gaussian_state, d):
+def strawberryfields_benchmark(benchmark, example_sf_gaussian_state, d, sf):
     @benchmark
     def func():
         new_program = sf.Program(d)
