@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ def gaussian_wigner_function(
     *,
     d: int,
     mean: np.ndarray,
-    cov: np.ndarray
+    cov: np.ndarray,
 ) -> np.ndarray:
     return np.array(
         [
