@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -44,3 +44,7 @@ class InvalidSimulation(PiquassoException):
 
 class NotImplementedCalculation(PiquassoException):
     """Raiesed when a calculation is not implemented."""
+
+
+class CVQNNException(PiquassoException):
+    """Exception raised from the `cvqnn` module."""
