@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,7 +48,7 @@ class Q:
     permits, e.g. when registering states.
 
     One could use the `all` keyword to indicate that the registered
-    :class:`~piquasso.api.instruction.Instruction` mashould be applied to all modes,
+    :class:`~piquasso.api.instruction.Instruction` should be applied to all modes,
     i.e.::
 
         with pq.Program() as program:
@@ -107,10 +107,10 @@ class Q:
             (Q): The current qumode.
         """
 
-        if _context.current_program is None:
+        if len(_context.program_stack) == 0:
             raise PiquassoException(
-                "The current program is None; therefore, it is not possible to register"
-                " the parameters on the given modes.\n"
+                "There is no `Program` instance available; it is not possible to"
+                " register the parameters on the specified modes.\n"
                 "Please use `Q(...) | ...` only inside `with pq.Program() as program:` "
                 "blocks, e.g.\n"
                 "\n"
@@ -118,7 +118,7 @@ class Q:
                 "    pq.Q(0, 1) | pq.Squeezing(r=0.5)"
             )
 
-        rhs._apply_to_program_on_register(_context.current_program, register=self)
+        rhs._apply_to_program_on_register(_context.program_stack[-1], register=self)
 
         return self
 
