@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,10 +36,28 @@ class Program(_mixins.DictMixin, _mixins.RegisterMixin, _mixins.CodeMixin):
         with pq.Program() as program:
             pq.Q() | pq.Vacuum()
 
-            pq.Q(0, 1) | pq.Squeezing(r=0.5)
+            pq.Q(0) | pq.Squeezing(r=0.5)
+
+            pq.Q(0, 1) | pq.Beamsplitter(theta=np.pi / 4)
 
         simulator = pq.GaussianSimulator(d=5)
         result = simulator.execute(program)
+
+    One may also create a Piquasso program by directly providing the set of instructions
+    as constructor argument::
+
+        program = pq.Program(
+            instructions=[
+                pq.Vacuum(),
+                pq.Squeezing(r=0.5).on_modes(0),
+                pq.Beamsplitter(theta=np.pi / 4).on_modes(0, 1),
+            ]
+        )
+        simulator = pq.GaussianSimulator(d=5)
+        result = simulator.execute(program)
+
+    :ivar instructions: The instructions in the program.
+
     """
 
     instructions: list
@@ -74,12 +92,12 @@ class Program(_mixins.DictMixin, _mixins.RegisterMixin, _mixins.CodeMixin):
             )
 
     def __enter__(self) -> "Program":
-        _context.current_program = self
+        _context.program_stack.append(self)
 
         return self
 
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
-        _context.current_program = None
+        _context.program_stack.pop()
 
     @classmethod
     def from_dict(cls, dict_: dict) -> "Program":
@@ -153,3 +171,6 @@ class Program(_mixins.DictMixin, _mixins.RegisterMixin, _mixins.CodeMixin):
             script += four_spaces + "pass"
 
         return script
+
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}(instructions={self.instructions})"
