@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ from typing import List, Mapping, Optional, Type
 
 import blackbird as bb
 
-from .. import Instruction
+from ..api.instruction import Instruction
 from ..api.exceptions import PiquassoException
 
 
