@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,9 @@
 """Global context variables."""
 
 import typing
-from typing import Optional
+from typing import List
 
 if typing.TYPE_CHECKING:
     from piquasso.api.program import Program
 
-current_program: Optional["Program"] = None
+program_stack: List["Program"] = []
