@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,15 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""
+Channels
+========
+
+The built-in channel instructions in Piquasso.
+"""
+
 import numpy as np
 
 from piquasso._math.linalg import is_positive_semidefinite, is_real_2n_by_2n
 from piquasso._math.symplectic import symplectic_form
 from piquasso._math.validations import all_in_interval
 
-from piquasso.core import _mixins
-
-from piquasso.api.calculator import BaseCalculator
 from piquasso.api.exceptions import InvalidParameter
 from piquasso.api.instruction import Gate
 
@@ -50,7 +54,7 @@ class DeterministicGaussianChannel(Gate):
 
     Note:
         Currently, this instruction can only be used along with
-        :class:`~piquasso._backends.gaussian.state.GaussianState`.
+        :class:`~piquasso._simulators.gaussian.state.GaussianState`.
     """
 
     def __init__(self, X: np.ndarray, Y: np.ndarray) -> None:
@@ -66,6 +70,12 @@ class DeterministicGaussianChannel(Gate):
             InvalidParameter: If the specified 'X' and/or 'Y' matrices are invalid.
         """
 
+        super().__init__(params=dict(X=X, Y=Y))
+
+    def _validate(self):
+        X = self.params["X"]
+        Y = self.params["Y"]
+
         if not is_real_2n_by_2n(X):
             raise InvalidParameter(
                 f"The parameter 'X' must be a real 2n-by-2n matrix: X={X}"
@@ -92,8 +102,6 @@ class DeterministicGaussianChannel(Gate):
                 "corresponding to Gaussian channels."
             )
 
-        super().__init__(params=dict(X=X, Y=Y))
-
 
 class Attenuator(Gate):
     r"""The attenuator channel.
@@ -113,7 +121,7 @@ class Attenuator(Gate):
 
     Note:
         Currently, this instruction can only be used along with
-        :class:`~piquasso._backends.gaussian.state.GaussianState`.
+        :class:`~piquasso._simulators.gaussian.state.GaussianState`.
     """
 
     def __init__(self, theta: float, mean_thermal_excitation: float = 0) -> None:
@@ -127,24 +135,31 @@ class Attenuator(Gate):
         Raises:
             InvalidParameter: If the specified mean thermal excitation is not positive.
         """
+        super().__init__(
+            params=dict(theta=theta, mean_thermal_excitation=mean_thermal_excitation),
+        )
 
+    def _validate(self):
+        mean_thermal_excitation = self.params["mean_thermal_excitation"]
         if mean_thermal_excitation < 0:
             raise InvalidParameter(
                 "The parameter 'mean_thermal_excitation' must be a positive real "
                 f"number: mean_thermal_excitation={mean_thermal_excitation}"
             )
 
-        X = np.cos(theta) * np.identity(2)
+    def _get_computed_params(self) -> dict:
+        X = np.cos(self.params["theta"]) * np.identity(2)
 
-        Y = (np.sin(theta) ** 2) * (2 * mean_thermal_excitation + 1) * np.identity(2)
-
-        super().__init__(
-            params=dict(theta=theta, mean_thermal_excitation=mean_thermal_excitation),
-            extra_params=dict(X=X, Y=Y),
+        Y = (
+            (np.sin(self.params["theta"]) ** 2)
+            * (2 * self.params["mean_thermal_excitation"] + 1)
+            * np.identity(2)
         )
 
+        return dict(X=X, Y=Y)
 
-class Loss(Gate, _mixins.ScalingMixin):
+
+class Loss(Gate):
     r"""Applies a loss channel to the state.
 
     The transmissivity is defined by :math:`t = \cos \theta`, where :math:`\theta` is
@@ -153,37 +168,22 @@ class Loss(Gate, _mixins.ScalingMixin):
     particle.
 
     Note:
-        - Currently, this instruction can only be used along with
-        :class:`~piquasso._backends.sampling.simulator.SamplingSimulator`.
-        - The parameter `transmissivity` is usually called `transmittance`.
+        Currently, this instruction can only be used along with
+        :class:`~piquasso._simulators.sampling.simulator.SamplingSimulator`.
+
+    Note:
+        The parameter `transmissivity` is usually called `transmittance`.
+
     """
 
+    NUMBER_OF_MODES = 1
+
     def __init__(self, transmissivity: np.ndarray) -> None:
         """
         Args:
-            transmissivity (numpy.ndarray): The transmissivity array.
+            transmissivity (numpy.ndarray): The transmissivity.
         """
-        super().__init__(
-            params=dict(transmissivity=transmissivity),
-            extra_params=dict(
-                transmissivity=np.atleast_1d(transmissivity),
-            ),
-        )
-
-    def _autoscale(self, calculator: BaseCalculator) -> None:
-        transmissivity = self._extra_params["transmissivity"]
-        if transmissivity is None or len(self.modes) == len(transmissivity):
-            pass
-        elif len(transmissivity) == 1:
-            self._extra_params["transmissivity"] = calculator.np.array(
-                [transmissivity[0]] * len(self.modes),
-                dtype=complex,
-            )
-        else:
-            raise InvalidParameter(
-                f"The channel {self} is not applicable to modes {self.modes} with the "
-                "specified parameters."
-            )
+        super().__init__(params=dict(transmissivity=transmissivity))
 
 
 class LossyInterferometer(Gate):
@@ -206,7 +206,7 @@ class LossyInterferometer(Gate):
 
     Note:
         Currently, this instruction can only be used along with
-        :class:`~piquasso._backends.sampling.simulator.SamplingSimulator`.
+        :class:`~piquasso._simulators.sampling.simulator.SamplingSimulator`.
 
     Raises:
         InvalidParameter:
