@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,12 @@
 # limitations under the License.
 
 """
+Measurements
+============
+
+All the built-in measurement instructions in Piquasso. Measurement instructions should
+be placed at the end of the Piquasso program.
+
 .. note::
 
    When multiple shots are specified and the evolution of the choses simulated state is
@@ -22,6 +28,8 @@
 
 """
 
+from typing import Tuple
+
 import numpy as np
 
 from piquasso.api.exceptions import InvalidParameter
@@ -44,7 +52,7 @@ class ParticleNumberMeasurement(Measurement):
 
     .. note::
 
-        When used with :class:`~piquasso._backends.gaussian.state.GaussianState`, the
+        When used with :class:`~piquasso._simulators.gaussian.state.GaussianState`, the
         state is not evolved, since that would be non-Gaussian.
 
     """
@@ -118,6 +126,15 @@ class GeneraldyneMeasurement(Measurement):
             )
         )
 
+    def _validate(self):
+        detection_covariance = self.params["detection_covariance"]
+
+        if not is_positive_semidefinite(detection_covariance + 1j * symplectic_form(1)):
+            raise InvalidParameter(
+                "The parameter 'detection_covariance' is invalid, since it doesn't "
+                "fulfill the Robertson-Schrödinger uncertainty relation."
+            )
+
 
 class HomodyneMeasurement(Measurement):
     r"""Homodyne measurement.
@@ -151,18 +168,16 @@ class HomodyneMeasurement(Measurement):
                 :class:`HeterodyneMeasurement`.
         """
 
-        super().__init__(
-            params=dict(
-                phi=phi,
-                z=z,
-            ),
-            extra_params=dict(
-                detection_covariance=np.array(
-                    [
-                        [z**2, 0],
-                        [0, (1 / z) ** 2],
-                    ]
-                ),
+        super().__init__(params=dict(phi=phi, z=z))
+
+    def _get_computed_params(self) -> dict:
+        z = self.params["z"]
+        return dict(
+            detection_covariance=np.array(
+                [
+                    [z**2, 0],
+                    [0, (1 / z) ** 2],
+                ]
             ),
         )
 
@@ -184,8 +199,114 @@ class HeterodyneMeasurement(Measurement):
     """
 
     def __init__(self) -> None:
+        super().__init__()
+
+    def _get_computed_params(self) -> dict:
+        return dict(detection_covariance=np.identity(2))
+
+
+class PostSelectPhotons(Measurement):
+    """Post-selection on detected photon numbers.
+
+    Example usage::
+
+        with pq.Program() as program:
+            pq.Q(all) | pq.StateVector([0, 1, 0]) * np.sqrt(0.2)
+            pq.Q(all) | pq.StateVector([1, 1, 0]) * np.sqrt(0.3)
+            pq.Q(all) | pq.StateVector([2, 1, 0]) * np.sqrt(0.5)
+
+            pq.Q(0) | pq.Phaseshifter(np.pi)
+
+            pq.Q(1, 2) | pq.Beamsplitter(np.pi / 8)
+            pq.Q(0, 1) | pq.Beamsplitter(1.1437)
+            pq.Q(1, 2) | pq.Beamsplitter(-np.pi / 8)
+
+            pq.Q(1, 2) | pq.PostSelectPhotons(photon_counts=(1, 0))
+
+        simulator = pq.PureFockSimulator(d=3, config=pq.Config(cutoff=4))
+
+        state = simulator.execute(program).state
+
+
+    Note:
+        The resulting state is not normalized. To normalize it, use
+        :meth:`~piquasso._simulators.fock.pure.state.PureFockState.normalize`. Moreover,
+        it is a state over the remaining modes, i.e., if `k` modes are post-selected,
+        then the resulting state will be defined on a system with `k` modes less.
+    """
+
+    def __init__(self, photon_counts: Tuple[int, ...]):
+        """
+        Args:
+            photon_counts (Tuple[int, ...]):
+                The desired photon numbers on the specified modes.
+        """
+
+        super().__init__(params=dict(photon_counts=photon_counts))
+
+
+class ImperfectPostSelectPhotons(Measurement):
+    r"""Post-selection on detected photon numbers.
+
+    Example usage::
+
+        P = np.array(
+            [
+                [1.0, 0.1, 0.2],
+                [0.0, 0.9, 0.2],
+                [0.0, 0.0, 0.6],
+            ],
+        )
+
+        with pq.Program() as program:
+            pq.Q(all) | pq.StateVector([0, 1, 0]) * np.sqrt(0.2)
+            pq.Q(all) | pq.StateVector([1, 1, 0]) * np.sqrt(0.3)
+            pq.Q(all) | pq.StateVector([2, 1, 0]) * np.sqrt(0.5)
+
+            pq.Q(0) | pq.Phaseshifter(np.pi)
+
+            pq.Q(1, 2) | pq.Beamsplitter(np.pi / 8)
+            pq.Q(0, 1) | pq.Beamsplitter(1.1437)
+            pq.Q(1, 2) | pq.Beamsplitter(-np.pi / 8)
+
+            pq.Q(1, 2) | pq.ImperfectPostSelectPhotons(
+                photon_counts=(1, 0),
+                detector_efficiency_matrix=P,
+            )
+
+        simulator = pq.PureFockSimulator(d=3, config=pq.Config(cutoff=4))
+
+        state = simulator.execute(program).state
+
+
+    The detector efficiency matrix :math:`P` is defined elementwise as
+
+    .. math::
+
+        P_{n m} = p(\text{detected photon count } = n| \text{actual photon count } = m)
+
+    Note:
+        The resulting state is not normalized. To normalize it, use
+        :meth:`~piquasso._simulators.fock.pure.state.PureFockState.normalize`. Moreover,
+        it is a state over the remaining modes, i.e., if `k` modes are post-selected,
+        then the resulting state will be defined on a system with `k` modes less.
+    """
+
+    def __init__(
+        self,
+        photon_counts: Tuple[int, ...],
+        detector_efficiency_matrix: "np.ndarray",
+    ):
+        """_summary_
+
+        Args:
+            photon_counts (Tuple[int, ...]):
+                The desired photon numbers on the specified modes.
+            detector_efficiency_matrix (np.ndarray): Detector efficiency matrix.
+        """
         super().__init__(
-            extra_params=dict(
-                detection_covariance=np.identity(2),
-            ),
+            params=dict(
+                photon_counts=photon_counts,
+                detector_efficiency_matrix=detector_efficiency_matrix,
+            )
         )
