@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Iterable
+"""
+Preparations
+============
+
+The built-in preparation instructions in Piquasso. Preparation instructions should
+be placed at the beginning of the Piquasso program.
+"""
+
+
+from typing import Dict, Iterable, Tuple, Optional
 
 import numpy as np
 
@@ -39,7 +48,7 @@ class Vacuum(Preparation):
     """
 
     def __init__(self) -> None:
-        pass
+        super().__init__()
 
 
 class Mean(Preparation):
@@ -54,7 +63,7 @@ class Mean(Preparation):
             ...
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.gaussian.state.GaussianState`.
+    :class:`~piquasso._simulators.gaussian.state.GaussianState`.
 
     Note:
         The mean vector is dependent on :math:`\hbar`, but the value of :math:`\hbar`
@@ -84,7 +93,7 @@ class Covariance(Preparation):
             ...
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.gaussian.state.GaussianState`.
+    :class:`~piquasso._simulators.gaussian.state.GaussianState`.
 
     Note:
         The covariance matrix is dependent on :math:`\hbar`, but the value of
@@ -122,19 +131,19 @@ class Thermal(Preparation):
     The power on vectors is defined as
 
     .. math::
-        \vec{a}^{\vec{b}} = \prod_{i=1}^d a_i^{b_i}
+        \vec{a}^{\vec{b}} = \prod_{i=1}^d a_i^{b_i}.
 
     In terms of Gaussian states, the mean vector and covariance matrix is
 
     .. math::
         \mu &= 0_{2d} \\
         \sigma &= \hbar (
-            2 \operatorname{\textit{diag}}(\operatorname{\textit{repeat}}
+            2 \operatorname{diag}(\operatorname{repeat}
                 (\overline{n}, 2)) + I_{2d \times 2d}
-        )
+        ).
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.gaussian.state.GaussianState`.
+    :class:`~piquasso._simulators.gaussian.state.GaussianState`.
     """
 
     def __init__(self, mean_photon_numbers: Iterable[float]) -> None:
@@ -145,18 +154,24 @@ class Thermal(Preparation):
         Raises:
             InvalidParameter: If the mean photon numbers are not positive real numbers.
         """
+
+        super().__init__(params=dict(mean_photon_numbers=mean_photon_numbers))
+
+    def _validate(self):
+        mean_photon_numbers = self.params["mean_photon_numbers"]
+
         if not all_real_and_positive(mean_photon_numbers):
             raise InvalidParameter(
                 "The mean photon numbers must be positive real numbers: "
                 f"mean_photon_numbers: {mean_photon_numbers}"
             )
 
+    def _get_computed_params(self) -> dict:
+        mean_photon_numbers = self.params["mean_photon_numbers"]
+
         cov = np.diag(2 * np.repeat(np.array(mean_photon_numbers), 2) + 1)
 
-        super().__init__(
-            params=dict(mean_photon_numbers=mean_photon_numbers),
-            extra_params=dict(cov=cov),
-        )
+        return dict(cov=cov)
 
 
 class StateVector(Preparation, _mixins.WeightMixin):
@@ -173,34 +188,74 @@ class StateVector(Preparation, _mixins.WeightMixin):
             ...
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.fock.pure.state.PureFockState`.
+    :class:`~piquasso._simulators.fock.pure.state.PureFockState`.
     """
 
     def __init__(
-        self, occupation_numbers: Iterable[int], coefficient: complex = 1.0
+        self,
+        occupation_numbers: Optional[Iterable[int]] = None,
+        fock_amplitude_map: Optional[Dict[Tuple[int, ...], complex]] = None,
+        coefficient: complex = 1.0,
     ) -> None:
         """
         Args:
-            occupation_numbers (Iterable[int]): The occupation numbers.
+            occupation_numbers (Iterable[int], optional): The occupation numbers.
+            fock_amplitude_map (Dict[Tuple[int, ...], complex], optional):
+                A mapping of occupation numbers to their corresponding amplitudes.
             coefficient (complex, optional):
                 The coefficient of the occupation number. Defaults to :math:`1.0`.
 
         Raises:
+            InvalidParameter:
+                If neither `occupation_numbers` nor `fock_amplitude_map` is provided.
+            InvalidParameter:
+                If both `occupation_numbers` and `fock_amplitude_map` are provided.
             InvalidState:
                 If the specified occupation numbers are not all natural numbers.
+            InvalidState:
+                If the keys in `fock_amplitude_map` are not all natural numbers.
         """
 
-        if not all_natural(occupation_numbers):
-            raise InvalidState(
-                f"Invalid occupation numbers: occupation_numbers={occupation_numbers}"
+        if occupation_numbers is None and fock_amplitude_map is None:
+            raise InvalidParameter(
+                "Either 'occupation_numbers' or 'fock_amplitude_map' must be provided."
             )
 
-        super().__init__(
-            params=dict(
-                occupation_numbers=tuple(occupation_numbers),
-                coefficient=coefficient,
-            ),
-        )
+        if occupation_numbers is not None and fock_amplitude_map is not None:
+            raise InvalidParameter(
+                "Only one of 'occupation_numbers' or 'fock_amplitude_map' "
+                "can be provided."
+            )
+
+        if occupation_numbers is not None:
+            if not all_natural(occupation_numbers):
+                raise InvalidState(
+                    f"Invalid occupation numbers: "
+                    f"occupation_numbers={occupation_numbers}\n"
+                    "Occupation numbers must contain non-negative integers."
+                )
+            super().__init__(
+                params=dict(
+                    occupation_numbers=tuple(occupation_numbers),
+                    coefficient=coefficient,
+                ),
+            )
+
+        if fock_amplitude_map is not None:
+            if not all(
+                all_natural(occupation) for occupation in fock_amplitude_map.keys()
+            ):
+                raise InvalidState(
+                    f"Invalid occupation numbers in "
+                    f"fock_amplitude_map: {fock_amplitude_map}\n"
+                    "Occupation numbers must contain non-negative integers."
+                )
+            super().__init__(
+                params=dict(
+                    fock_amplitude_map=fock_amplitude_map,
+                    coefficient=coefficient,
+                ),
+            )
 
 
 class DensityMatrix(Preparation, _mixins.WeightMixin):
@@ -220,7 +275,7 @@ class DensityMatrix(Preparation, _mixins.WeightMixin):
         This only creates one matrix element.
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.fock.general.state.FockState`.
+    :class:`~piquasso._simulators.fock.general.state.FockState`.
     """
 
     def __init__(
@@ -271,12 +326,12 @@ class Create(Preparation):
             ...
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.fock.general.state.FockState`,
-    :class:`~piquasso._backends.fock.pure.state.PureFockState`.
+    :class:`~piquasso._simulators.fock.general.state.FockState`,
+    :class:`~piquasso._simulators.fock.pure.state.PureFockState`.
     """
 
     def __init__(self) -> None:
-        pass
+        super().__init__()
 
 
 class Annihilate(Preparation):
@@ -293,9 +348,9 @@ class Annihilate(Preparation):
             ...
 
     Can only be applied to the following states:
-    :class:`~piquasso._backends.fock.general.state.FockState`,
-    :class:`~piquasso._backends.fock.pure.state.PureFockState`.
+    :class:`~piquasso._simulators.fock.general.state.FockState`,
+    :class:`~piquasso._simulators.fock.pure.state.PureFockState`.
     """
 
     def __init__(self) -> None:
-        pass
+        super().__init__()
