@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2023 Budapest Quantum Computing Group
+# Copyright 2021-2025 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,3 +22,9 @@ def test_program_copy():
     program_copy = program.copy()
 
     assert program_copy is not program
+
+
+def test_program_repr():
+    program = pq.Program()
+
+    assert repr(program) == "Program(instructions=[])"
